@@ -19,7 +19,11 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-from future_builtins import filter, map
+from builtins import map
+from builtins import filter
+from builtins import next
+from builtins import range
+from builtins import object
 
 import re  # @UnusedImport
 import types
@@ -37,7 +41,7 @@ from duplicity import util  # @UnusedImport
 from duplicity.path import *  # @UnusedWildImport
 from duplicity.lazy import *  # @UnusedWildImport
 
-"""Functions for patching of directories"""
+u"""Functions for patching of directories"""
 
 
 class PatchDirException(Exception):
@@ -45,20 +49,20 @@ class PatchDirException(Exception):
 
 
 def Patch(base_path, difftar_fileobj):
-    """Patch given base_path and file object containing delta"""
-    diff_tarfile = tarfile.TarFile("arbitrary", "r", difftar_fileobj)
+    u"""Patch given base_path and file object containing delta"""
+    diff_tarfile = tarfile.TarFile(u"arbitrary", u"r", difftar_fileobj)
     patch_diff_tarfile(base_path, diff_tarfile)
     assert not difftar_fileobj.close()
 
 
 def Patch_from_iter(base_path, fileobj_iter, restrict_index=()):
-    """Patch given base_path and iterator of delta file objects"""
+    u"""Patch given base_path and iterator of delta file objects"""
     diff_tarfile = TarFile_FromFileobjs(fileobj_iter)
     patch_diff_tarfile(base_path, diff_tarfile, restrict_index)
 
 
 def patch_diff_tarfile(base_path, diff_tarfile, restrict_index=()):
-    """Patch given Path object using delta tarfile (as in tarfile.TarFile)
+    u"""Patch given Path object using delta tarfile (as in tarfile.TarFile)
 
     If restrict_index is set, ignore any deltas in diff_tarfile that
     don't start with restrict_index.
@@ -77,12 +81,12 @@ def patch_diff_tarfile(base_path, diff_tarfile, restrict_index=()):
     ITR = IterTreeReducer(PathPatcher, [base_path])
     for basis_path, diff_ropath in collated:
         if basis_path:
-            log.Info(_("Patching %s") % (util.ufn(basis_path.get_relative_path())),
+            log.Info(_(u"Patching %s") % (util.fsdecode(basis_path.get_relative_path())),
                      log.InfoCode.patch_file_patching,
                      util.escape(basis_path.get_relative_path()))
             ITR(basis_path.index, basis_path, diff_ropath)
         else:
-            log.Info(_("Patching %s") % (util.ufn(diff_ropath.get_relative_path())),
+            log.Info(_(u"Patching %s") % (util.fsdecode(diff_ropath.get_relative_path())),
                      log.InfoCode.patch_file_patching,
                      util.escape(diff_ropath.get_relative_path()))
             ITR(diff_ropath.index, basis_path, diff_ropath)
@@ -96,7 +100,7 @@ def empty_iter():
 
 
 def filter_path_iter(path_iter, index):
-    """Rewrite path elements of path_iter so they start with index
+    u"""Rewrite path elements of path_iter so they start with index
 
     Discard any that doesn't start with index, and remove the index
     prefix from the rest.
@@ -111,19 +115,22 @@ def filter_path_iter(path_iter, index):
 
 
 def difftar2path_iter(diff_tarfile):
-    """Turn file-like difftarobj into iterator of ROPaths"""
+    u"""Turn file-like difftarobj into iterator of ROPaths"""
     tar_iter = iter(diff_tarfile)
     multivol_fileobj = None
 
     # The next tar_info is stored in this one element list so
     # Multivol_Filelike below can update it.  Any StopIterations will
     # be passed upwards.
-    tarinfo_list = [tar_iter.next()]
+    try:
+        tarinfo_list = [next(tar_iter)]
+    except StopIteration:
+        return
 
     while 1:
         # This section relevant when a multivol diff is last in tar
         if not tarinfo_list[0]:
-            raise StopIteration
+            return
         if multivol_fileobj and not multivol_fileobj.at_end:
             multivol_fileobj.close()  # aborting in middle of multivol
             continue
@@ -132,7 +139,7 @@ def difftar2path_iter(diff_tarfile):
         ropath = ROPath(index)
         ropath.init_from_tarinfo(tarinfo_list[0])
         ropath.difftype = difftype
-        if difftype == "deleted":
+        if difftype == u"deleted":
             ropath.type = None
         elif ropath.isreg():
             if multivol:
@@ -144,65 +151,73 @@ def difftar2path_iter(diff_tarfile):
             else:
                 ropath.setfileobj(diff_tarfile.extractfile(tarinfo_list[0]))
         yield ropath
-        tarinfo_list[0] = tar_iter.next()
+        try:
+            tarinfo_list[0] = next(tar_iter)
+        except StopIteration:
+            return
 
 
 def get_index_from_tarinfo(tarinfo):
-    """Return (index, difftype, multivol) pair from tarinfo object"""
-    for prefix in ["snapshot/", "diff/", "deleted/",
-                   "multivol_diff/", "multivol_snapshot/"]:
+    u"""Return (index, difftype, multivol) pair from tarinfo object"""
+    for prefix in [u"snapshot/", u"diff/", u"deleted/",
+                   u"multivol_diff/", u"multivol_snapshot/"]:
         tiname = util.get_tarinfo_name(tarinfo)
+        if sys.version_info.major == 2 and isinstance(prefix, unicode):
+            prefix = prefix.encode()
         if tiname.startswith(prefix):
             name = tiname[len(prefix):]  # strip prefix
-            if prefix.startswith("multivol"):
-                if prefix == "multivol_diff/":
-                    difftype = "diff"
+            if prefix.startswith(u"multivol"):
+                if prefix == u"multivol_diff/":
+                    difftype = u"diff"
                 else:
-                    difftype = "snapshot"
+                    difftype = u"snapshot"
                 multivol = 1
                 name, num_subs = \
-                    re.subn("(?s)^multivol_(diff|snapshot)/?(.*)/[0-9]+$",
-                            "\\2", tiname)
+                    re.subn(u"(?s)^multivol_(diff|snapshot)/?(.*)/[0-9]+$",
+                            u"\\2", tiname)
                 if num_subs != 1:
                     raise PatchDirException(u"Unrecognized diff entry %s" %
-                                            util.ufn(tiname))
+                                            tiname)
             else:
                 difftype = prefix[:-1]  # strip trailing /
                 name = tiname[len(prefix):]
-                if name.endswith("/"):
+                if name.endswith(r"/"):
                     name = name[:-1]  # strip trailing /'s
                 multivol = 0
             break
     else:
         raise PatchDirException(u"Unrecognized diff entry %s" %
-                                util.ufn(tiname))
-    if name == "." or name == "":
+                                tiname)
+    if name == r"." or name == r"":
         index = ()
     else:
-        index = tuple(name.split("/"))
-        if '..' in index:
+        if sys.version_info.major >= 3:
+            index = tuple(util.fsencode(name).split(b"/"))
+        else:
+            index = tuple(name.split(b"/"))
+        if b'..' in index:
             raise PatchDirException(u"Tar entry %s contains '..'.  Security "
-                                    "violation" % util.ufn(tiname))
+                                    u"violation" % util.fsdecode(tiname))
     return (index, difftype, multivol)
 
 
-class Multivol_Filelike:
-    """Emulate a file like object from multivols
+class Multivol_Filelike(object):
+    u"""Emulate a file like object from multivols
 
     Maintains a buffer about the size of a volume.  When it is read()
     to the end, pull in more volumes as desired.
 
     """
     def __init__(self, tf, tar_iter, tarinfo_list, index):
-        """Initializer.  tf is TarFile obj, tarinfo is first tarinfo"""
+        u"""Initializer.  tf is TarFile obj, tarinfo is first tarinfo"""
         self.tf, self.tar_iter = tf, tar_iter
         self.tarinfo_list = tarinfo_list  # must store as list for write access
         self.index = index
-        self.buffer = ""
+        self.buffer = b""
         self.at_end = 0
 
     def read(self, length=-1):
-        """Read length bytes from file"""
+        u"""Read length bytes from file"""
         if length < 0:
             while self.addtobuffer():
                 pass
@@ -218,7 +233,7 @@ class Multivol_Filelike:
         return result
 
     def addtobuffer(self):
-        """Add next chunk to buffer"""
+        u"""Add next chunk to buffer"""
         if self.at_end:
             return None
         index, difftype, multivol = get_index_from_tarinfo(  # @UnusedVariable
@@ -234,7 +249,7 @@ class Multivol_Filelike:
         fp.close()
 
         try:
-            self.tarinfo_list[0] = self.tar_iter.next()
+            self.tarinfo_list[0] = next(self.tar_iter)
         except StopIteration:
             self.tarinfo_list[0] = None
             self.at_end = 1
@@ -242,24 +257,24 @@ class Multivol_Filelike:
         return 1
 
     def close(self):
-        """If not at end, read remaining data"""
+        u"""If not at end, read remaining data"""
         if not self.at_end:
             while 1:
-                self.buffer = ""
+                self.buffer = b""
                 if not self.addtobuffer():
                     break
         self.at_end = 1
 
 
 class PathPatcher(ITRBranch):
-    """Used by DirPatch, process the given basis and diff"""
+    u"""Used by DirPatch, process the given basis and diff"""
     def __init__(self, base_path):
-        """Set base_path, Path of root of tree"""
+        u"""Set base_path, Path of root of tree"""
         self.base_path = base_path
         self.dir_diff_ropath = None
 
     def start_process(self, index, basis_path, diff_ropath):
-        """Start processing when diff_ropath is a directory"""
+        u"""Start processing when diff_ropath is a directory"""
         if not (diff_ropath and diff_ropath.isdir()):
             assert index == (), util.uindex(index)  # should only happen for first elem
             self.fast_process(index, basis_path, diff_ropath)
@@ -276,44 +291,44 @@ class PathPatcher(ITRBranch):
         self.dir_diff_ropath = diff_ropath
 
     def end_process(self):
-        """Copy directory permissions when leaving tree"""
+        u"""Copy directory permissions when leaving tree"""
         if self.dir_diff_ropath:
             self.dir_diff_ropath.copy_attribs(self.dir_basis_path)
 
     def can_fast_process(self, index, basis_path, diff_ropath):
-        """No need to recurse if diff_ropath isn't a directory"""
+        u"""No need to recurse if diff_ropath isn't a directory"""
         return not (diff_ropath and diff_ropath.isdir())
 
     def fast_process(self, index, basis_path, diff_ropath):
-        """For use when neither is a directory"""
+        u"""For use when neither is a directory"""
         if not diff_ropath:
             return  # no change
         elif not basis_path:
-            if diff_ropath.difftype == "deleted":
+            if diff_ropath.difftype == u"deleted":
                 pass  # already deleted
             else:
                 # just copy snapshot over
                 diff_ropath.copy(self.base_path.new_index(index))
-        elif diff_ropath.difftype == "deleted":
+        elif diff_ropath.difftype == u"deleted":
             if basis_path.isdir():
                 basis_path.deltree()
             else:
                 basis_path.delete()
-        elif not basis_path.isreg() or (basis_path.isreg() and diff_ropath.difftype == "snapshot"):
+        elif not basis_path.isreg() or (basis_path.isreg() and diff_ropath.difftype == u"snapshot"):
             if basis_path.isdir():
                 basis_path.deltree()
             else:
                 basis_path.delete()
             diff_ropath.copy(basis_path)
         else:
-            assert diff_ropath.difftype == "diff", diff_ropath.difftype
+            assert diff_ropath.difftype == u"diff", diff_ropath.difftype
             basis_path.patch_with_attribs(diff_ropath)
 
 
-class TarFile_FromFileobjs:
-    """Like a tarfile.TarFile iterator, but read from multiple fileobjs"""
+class TarFile_FromFileobjs(object):
+    u"""Like a tarfile.TarFile iterator, but read from multiple fileobjs"""
     def __init__(self, fileobj_iter):
-        """Make new tarinfo iterator
+        u"""Make new tarinfo iterator
 
         fileobj_iter should be an iterator of file objects opened for
         reading.  They will be closed at end of reading.
@@ -323,34 +338,37 @@ class TarFile_FromFileobjs:
         self.tarfile, self.tar_iter = None, None
         self.current_fp = None
 
-    def __iter__(self):
+    def __iter__(self):  # pylint: disable=non-iterator-returned
         return self
 
     def set_tarfile(self):
-        """Set tarfile from next file object, or raise StopIteration"""
+        u"""Set tarfile from next file object, or raise StopIteration"""
         if self.current_fp:
             assert not self.current_fp.close()
-        self.current_fp = self.fileobj_iter.next()
-        self.tarfile = util.make_tarfile("r", self.current_fp)
+        self.current_fp = next(self.fileobj_iter)
+        self.tarfile = util.make_tarfile(u"r", self.current_fp)
         self.tar_iter = iter(self.tarfile)
 
-    def next(self):
+    def __next__(self):
         if not self.tarfile:
-            self.set_tarfile()
+            try:
+                self.set_tarfile()
+            except StopIteration:
+                return
         try:
-            return self.tar_iter.next()
+            return next(self.tar_iter)
         except StopIteration:
             assert not self.tarfile.close()
             self.set_tarfile()
-            return self.tar_iter.next()
+            return next(self.tar_iter)
 
     def extractfile(self, tarinfo):
-        """Return data associated with given tarinfo"""
+        u"""Return data associated with given tarinfo"""
         return self.tarfile.extractfile(tarinfo)
 
 
 def collate_iters(iter_list):
-    """Collate iterators by index
+    u"""Collate iterators by index
 
     Input is a list of n iterators each of which must iterate elements
     with an index attribute.  The elements must come out in increasing
@@ -371,18 +389,18 @@ def collate_iters(iter_list):
     elems = overflow[:]
 
     def setrorps(overflow, elems):
-        """Set the overflow and rorps list"""
+        u"""Set the overflow and rorps list"""
         for i in range(iter_num):
             if not overflow[i] and elems[i] is None:
                 try:
-                    elems[i] = iter_list[i].next()
+                    elems[i] = next(iter_list[i])
                 except StopIteration:
                     overflow[i] = 1
                     elems[i] = None
 
     def getleastindex(elems):
-        """Return the first index in elems, assuming elems isn't empty"""
-        return min(map(lambda elem: elem.index, filter(lambda x: x, elems)))
+        u"""Return the first index in elems, assuming elems isn't empty"""
+        return min([elem.index for elem in [x for x in elems if x]])
 
     def yield_tuples(iter_num, overflow, elems):
         while 1:
@@ -402,8 +420,8 @@ def collate_iters(iter_list):
     return yield_tuples(iter_num, overflow, elems)
 
 
-class IndexedTuple:
-    """Like a tuple, but has .index (used previously by collate_iters)"""
+class IndexedTuple(object):
+    u"""Like a tuple, but has .index (used previously by collate_iters)"""
     def __init__(self, index, sequence):
         self.index = index
         self.data = tuple(sequence)
@@ -412,7 +430,7 @@ class IndexedTuple:
         return len(self.data)
 
     def __getitem__(self, key):
-        """This only works for numerical keys (easier this way)"""
+        u"""This only works for numerical keys (easier this way)"""
         return self.data[key]
 
     def __lt__(self, other):
@@ -442,17 +460,17 @@ class IndexedTuple:
     def __eq__(self, other):
         if isinstance(other, IndexedTuple):
             return self.index == other.index and self.data == other.data
-        elif isinstance(other, types.TupleType):
+        elif isinstance(other, tuple):
             return self.data == other
         else:
             return None
 
     def __str__(self):
-        return "(%s).%s" % (", ".join(map(str, self.data)), self.index)
+        return u"(%s).%s" % (u", ".join(map(str, self.data)), self.index)
 
 
 def normalize_ps(patch_sequence):
-    """Given an sequence of ROPath deltas, remove blank and unnecessary
+    u"""Given an sequence of ROPath deltas, remove blank and unnecessary
 
     The sequence is assumed to be in patch order (later patches apply
     to earlier ones).  A patch is unnecessary if a later one doesn't
@@ -467,52 +485,52 @@ def normalize_ps(patch_sequence):
         if delta is not None:
             # skip blank entries
             result_list.insert(0, delta)
-            if delta.difftype != "diff":
+            if delta.difftype != u"diff":
                 break
         i -= 1
     return result_list
 
 
 def patch_seq2ropath(patch_seq):
-    """Apply the patches in patch_seq, return single ropath"""
+    u"""Apply the patches in patch_seq, return single ropath"""
     first = patch_seq[0]
-    assert first.difftype != "diff", "First patch in sequence " \
-                                     "%s was a diff" % patch_seq
+    assert first.difftype != u"diff", u"First patch in sequence " \
+                                      u"%s was a diff" % patch_seq
     if not first.isreg():
         # No need to bother with data if not regular file
-        assert len(patch_seq) == 1, "Patch sequence isn't regular, but " \
-                                    "has %d entries" % len(patch_seq)
+        assert len(patch_seq) == 1, u"Patch sequence isn't regular, but " \
+                                    u"has %d entries" % len(patch_seq)
         return first.get_ropath()
 
-    current_file = first.open("rb")
+    current_file = first.open(u"rb")
 
     for delta_ropath in patch_seq[1:]:
-        assert delta_ropath.difftype == "diff", delta_ropath.difftype
-        if not isinstance(current_file, file):
-            """
+        assert delta_ropath.difftype == u"diff", delta_ropath.difftype
+        try:
+            cur_file.fileno()
+        except:
+            u"""
             librsync insists on a real file object, which we create manually
             by using the duplicity.tempdir to tell us where.
 
             See https://bugs.launchpad.net/duplicity/+bug/670891 for discussion
-            of os.tmpfile() vs tempfile.TemporaryFile() w.r.t. Windows / Linux.
+            of os.tmpfile() vs tempfile.TemporaryFile() w.r.t. Windows / Posix,
+            which is worked around in librsync.PatchedFile() now.
             """
-            if sys.platform.startswith(('cygwin', 'windows')):
-                tempfp = os.tmpfile()
-            else:
-                tempfp = tempfile.TemporaryFile(dir=tempdir.default().dir())
+            tempfp = tempfile.TemporaryFile(dir=tempdir.default().dir())
             util.copyfileobj(current_file, tempfp)
             assert not current_file.close()
             tempfp.seek(0)
             current_file = tempfp
         current_file = librsync.PatchedFile(current_file,
-                                            delta_ropath.open("rb"))
+                                            delta_ropath.open(u"rb"))
     result = patch_seq[-1].get_ropath()
     result.setfileobj(current_file)
     return result
 
 
 def integrate_patch_iters(iter_list):
-    """Combine a list of iterators of ropath patches
+    u"""Combine a list of iterators of ropath patches
 
     The iter_list should be sorted in patch order, and the elements in
     each iter_list need to be orderd by index.  The output will be an
@@ -529,14 +547,14 @@ def integrate_patch_iters(iter_list):
                 yield final_ropath
         except Exception as e:
             filename = normalized[-1].get_ropath().get_relative_path()
-            log.Warn(_("Error '%s' patching %s") %
-                     (util.uexc(e), util.ufn(filename)),
+            log.Warn(_(u"Error '%s' patching %s") %
+                     (util.uexc(e), util.fsdecode(filename)),
                      log.WarningCode.cannot_process,
                      util.escape(filename))
 
 
 def tarfiles2rop_iter(tarfile_list, restrict_index=()):
-    """Integrate tarfiles of diffs into single ROPath iter
+    u"""Integrate tarfiles of diffs into single ROPath iter
 
     Then filter out all the diffs in that index which don't start with
     the restrict_index.
@@ -550,7 +568,7 @@ def tarfiles2rop_iter(tarfile_list, restrict_index=()):
 
 
 def Write_ROPaths(base_path, rop_iter):
-    """Write out ropaths in rop_iter starting at base_path
+    u"""Write out ropaths in rop_iter starting at base_path
 
     Returns 1 if something was actually written, 0 otherwise.
 
@@ -566,20 +584,20 @@ def Write_ROPaths(base_path, rop_iter):
 
 
 class ROPath_IterWriter(ITRBranch):
-    """Used in Write_ROPaths above
+    u"""Used in Write_ROPaths above
 
     We need to use an ITR because we have to update the
     permissions/times of directories after we write the files in them.
 
     """
     def __init__(self, base_path):
-        """Set base_path, Path of root of tree"""
+        u"""Set base_path, Path of root of tree"""
         self.base_path = base_path
         self.dir_diff_ropath = None
         self.dir_new_path = None
 
     def start_process(self, index, ropath):
-        """Write ropath.  Only handles the directory case"""
+        u"""Write ropath.  Only handles the directory case"""
         if not ropath.isdir():
             # Base may not be a directory, but rest should
             assert ropath.index == (), ropath.index
@@ -598,19 +616,19 @@ class ROPath_IterWriter(ITRBranch):
         self.dir_diff_ropath = ropath
 
     def end_process(self):
-        """Update information of a directory when leaving it"""
+        u"""Update information of a directory when leaving it"""
         if self.dir_diff_ropath:
             self.dir_diff_ropath.copy_attribs(self.dir_new_path)
 
     def can_fast_process(self, index, ropath):
-        """Can fast process (no recursion) if ropath isn't a directory"""
-        log.Info(_("Writing %s of type %s") %
-                 (util.ufn(ropath.get_relative_path()), ropath.type),
+        u"""Can fast process (no recursion) if ropath isn't a directory"""
+        log.Info(_(u"Writing %s of type %s") %
+                 (util.fsdecode(ropath.get_relative_path()), ropath.type),
                  log.InfoCode.patch_file_writing,
-                 "%s %s" % (util.escape(ropath.get_relative_path()), ropath.type))
+                 u"%s %s" % (util.escape(ropath.get_relative_path()), ropath.type))
         return not ropath.isdir()
 
     def fast_process(self, index, ropath):
-        """Write non-directory ropath to destination"""
+        u"""Write non-directory ropath to destination"""
         if ropath.exists():
             ropath.copy(self.base_path.new_index(index))
